@@ -15,23 +15,20 @@ class R3sponse(object):
 	def __init__(self):	
 
 		# set log file.
-		self.log_file = None
+		#	assign new values with self.log_file.assign(...) to keep it shared across other objects.
+		self.log_file = String(str(None))
 
 		# set log level.
-		self.log_level = 0
-		#if utils.argument_present('--log-level'):
-		#	try: self.log_level = int(utils.get_argument('--log-level'))
-		#	except: self.log_level = 0
-		#
+		#	assign new values with self.log_level.assign(...) to keep it shared across other objects.
+		self.log_level = Integer(0)
+		
+		# objects.
+		self.parameters = Parameters()
+
+		# for imports.
 		self.ResponseObject = ResponseObject
 
 	# response functions.
-	def default_response(self, attributes={
-		"success":False,
-		"message":None,
-		"error":None,
-	}):
-		return self.ResponseObject(attributes=attributes)
 	def success(self,
 		# the message (must be param #1).
 		message, 
@@ -46,7 +43,7 @@ class R3sponse(object):
 		# return as a django JsonResponse.
 		django=False,
 	):
-		response = self.default_response({
+		response = self.response({
 			"success":True,
 			"message":message,
 			"error":None,
@@ -74,7 +71,7 @@ class R3sponse(object):
 		# raise error for developer traceback.
 		traceback=ERROR_TRACEBACK,
 	):
-		response = self.default_response({
+		response = self.response({
 			"success":False,
 			"message":None,
 			"error":error,
@@ -90,84 +87,6 @@ class R3sponse(object):
 		return response
 		#
 
-	# parameter functions.
-	def get_request_parameter(self, request, identifier, traceback=None):
-		response = self.default_response()
-		format = None
-		if ":" in identifier:
-			identifier,format = identifier.split(":")
-			while True:
-				if " " in format: format = format.replace(" ","")
-				else: break
-		if request.method in ["post", "POST"]:
-			variable = request.POST.get(identifier)
-		else:
-			variable = request.GET.get(identifier)
-		if variable in ["", None]:
-			if traceback != None:
-				return variable, self.error(f"{traceback}: Define parameter: [{identifier}].")
-			else:
-				return variable, self.error(f"Define parameter: [{identifier}].")
-
-		elif format != None:
-			if format in ["str", "string"]: variable = str(variable)
-			elif format in ["int", "integer"]: variable = int(variable)
-			elif format in ["bool", "boolean"]: 
-				if variable in ["true", "True", "TRUE", True]: variable = True
-				else: variable = False
-			elif format in ["float", "double"]: variable = float(variable)
-			elif format in ["array", "list"]: variable = variable.split(",")
-			else:
-				raise ValueError(f"Unrecognized <r3sponse.get_request_parameter> format: {format}.")
-		return variable, self.success(f"Succesfully retrieved request parameter [{identifier}].", {
-			"key":identifier,
-			"value":variable,
-		})
-	def get_request_parameters(self, request, identifiers=[], optional=False, empty_value=None, traceback=None):
-		if isinstance(identifiers, str):
-			return self.error("__get_request_params__ is used to retrieve several identifiers (array format not string).")
-		params = self.ResponseObject()
-		for param in identifiers:
-			param_value, response = self.get_request_parameter(request, param, traceback=traceback)
-			param = param.split(":")[0]
-			if response["error"] != None: 
-				if optional:
-					params[param] = empty_value
-				else:
-					return params, response
-			else: 
-				params[param] = param_value
-		if optional:
-			for key in identifiers:
-				try: params[key]
-				except: params[key] = empty_value
-		return params, self.success(f"Succesfully retrieved {len(params)} request parameter(s).")
-
-	# check parameters.
-	def check_parameter(self, parameter=None, name="parameter", empty_value=None, traceback=None):
-		response = self.default_response()
-		if parameter == empty_value: 
-			if traceback != None:
-				return self.error(f"{traceback}: Define parameter [{name}].")
-			else:
-				return self.error(f"Define parameter [{name}].")
-		if ":" in name:
-			name,formats = name.split(":")
-			while True:
-				if " " in formats: formats = formats.replace(" ","")
-				else: break
-			formats = formats.split(",")
-			param_format = Formats.get(parameter, serialize=True)
-			if param_format not in formats:
-				return self.error(f"Incorrect parameter [{name}:{param_format}] format, correct format(s): {Array(path=False, array=formats).string(joiner=', ')}.")
-		return self.success(f"Succesfully checked parameter [{name}].")
-	def check_parameters(self, parameters={"parameter":None}, empty_value=None, traceback=None):
-		response = self.default_response()
-		for id, value in parameters.items():
-			response = self.check_parameter(value, id, empty_value=empty_value, traceback=traceback)
-			if response["error"] != None: return response
-		return response
-
 	# log functions.
 	def log(self, 
 		# option 1:
@@ -238,7 +157,7 @@ class R3sponse(object):
 		#
 	def load_logs(self, format="webserver", options=["webserver", "cli", "array", "string"]):
 		try:
-			logs = Formats.File(self.log_file, load=True, blank="").data
+			logs = File(str(self.log_file), load=True, blank="").data
 		except:
 			return self.error("Failed to load the logs.")
 		if format == "webserver":
@@ -254,33 +173,11 @@ class R3sponse(object):
 
 		return self.success("Succesfully loaded the logs.", {"logs":logs})
 	def reset_logs(self):
-		Formats.File(self.log_file).save(f"Resetted log file.\n")
+		Formats.File(str(self.log_file)).save(f"Resetted log file.\n")
+		
 		#
 	
-	# system functions.
-	def __log_to_file__(self, message):
-
-		# init.
-		response = self.default_response()
-		try:
-			with open(self.log_file, "a") as file:
-				file.write(f'{Date().seconds_timestamp} - {message}\n')
-			response["success"] = True
-			response["message"] = "Succesfully logged the message."
-		except:
-			response["error"] = "Failed to log the message."
-			return response
-		
-		# check file size.
-		size = FilePath(self.log_file).size(mode="mb", format="integer")
-		if size >= 100: self.reset_logs()
-
-		# return response.
-		return response
-
-		#
-
-	# a response object.
+	# init a response blank object.
 	def serialize(self, response={}):
 		if isinstance(response, str):
 			try:
@@ -314,20 +211,202 @@ class R3sponse(object):
 					except: a=1
 			response[key] = value
 		return response
-	def response(self, response={
+	def response(self, 
+		# the blank response (dict) (#1).
+		response={
 			"success":False,
 			"message":None,
 			"error":None,
-	}):
-		return self.ResponseObject(response)
-	def safe_response(self, response={
-			"success":False,
-			"message":None,
-			"error":None,
-	}):
-		return self.response(self.serialize(response))
+		},
+		# the safe boolean (serialize from dumped json).
+		safe=False,
+	):
+		# do not serialize (always keep default).
+		if not safe:
+			return ResponseObject(response)
 
-# a response object.
+		# serialize the response (serialize from dumped json).
+		else:
+			return self.response(self.serialize(response))
+
+		#
+
+	# system functions.
+	def __log_to_file__(self, message):
+
+		# init.
+		try:
+			with open(self.log_file, "a") as file:
+				file.write(f'{Date().seconds_timestamp} - {message}\n')
+			response = self.response()
+			response["success"] = True
+			response["message"] = "Succesfully logged the message."
+		
+			# check file size.
+			size = FilePath(self.log_file).size(mode="mb", format="integer")
+			if size >= 100: self.reset_logs()
+
+			# handler.
+			return response
+
+		except Exception as e:
+			response = self.response()
+			response["error"] = f"Failed to log the message, error: {e}."
+			return response
+			
+		#
+
+	#
+# the parameters manager object class.
+class Parameters(object):
+	def __init__(self):
+		a=1
+
+	# get request parameters.
+	def get(self, 
+		# the django request (1).
+		request=None, 
+		# the identifiers (#2).
+		#	str instance: return the parameters value.
+		#	list instance: return a parameters object class & return an error response when a parameter is undefined.
+		#	dict instance: return a parameters object class & return the parameter's value from the dict as a default when undefined.
+		parameters=[], 
+		# default return value (dict instance of parameters overwrites the default parameter).
+		default=None, 
+		# traceback id.
+		traceback=None,
+	):
+		if request == None:
+			raise exceptions.InvalidUsage("<r3sponse.paramters.get>: Define parameter: [request].")
+		
+		# single parameter.
+		if isinstance(parameters, (str,String)):
+			parameters = str(parameters)
+			response = r3sponse.response()
+			format = None
+			if ":" in parameters:
+				parameters,format = parameters.split(":")
+				while True:
+					if " " in format: format = format.replace(" ","")
+					else: break
+			if request.method in ["post", "POST"]:
+				variable = request.POST.get(parameters)
+			else:
+				variable = request.GET.get(parameters)
+			if variable in ["", None]:
+				if traceback != None:
+					return variable, r3sponse.error(f"{traceback}: Define parameter: [{parameters}].")
+				else:
+					return variable, r3sponse.error(f"Define parameter: [{parameters}].")
+			elif format != None:
+				if format in ["str", "string"]: variable = str(variable)
+				elif format in ["int", "integer"]: variable = int(variable)
+				elif format in ["bool", "boolean"]: 
+					if variable in ["true", "True", "TRUE", True]: variable = True
+					else: variable = False
+				elif format in ["float", "double"]: variable = float(variable)
+				elif format in ["array", "list"]: variable = variable.split(",")
+				else:
+					raise ValueError(f"Unrecognized <r3sponse.parameters.get> format: {format}.")
+			
+			# handler.
+			return variable, r3sponse.success(f"Succesfully retrieved request parameter [{parameters}].", {
+				"key":parameters,
+				"value":variable,
+			})
+
+		# list recursive.
+		elif isinstance(parameters, (list, Array)):
+			optional = False
+			params = ResponseObject()
+			for param in parameters:
+				param_value, response = self.get(request, param, traceback=traceback)
+				param = param.split(":")[0]
+				if response["error"] != None: 
+					if optional:
+						params[param] = default
+					else:
+						return params, response
+				else: 
+					params[param] = param_value
+			if optional:
+				for key in parameters:
+					try: params[key]
+					except: params[key] = default
+			return params, r3sponse.success(f"Succesfully retrieved {len(params)} request parameter(s).")
+
+		# dict recursive.
+		elif isinstance(parameters, (dict, Dictionary, ResponseObject)):
+			if isinstance(parameters, (ResponseObject)): parameters = parameters.clean()
+			optional = True
+			params = ResponseObject()
+			for param,default in parameters.items():
+				param_value, response = self.get(request, param, traceback=traceback)
+				param = param.split(":")[0]
+				if response["error"] != None: 
+					if optional:
+						params[param] = default
+					else:
+						return params, response
+				else: 
+					params[param] = param_value
+			if optional:
+				for key,default in parameters.items():
+					try: params[key]
+					except: params[key] = default
+			return params, r3sponse.success(f"Succesfully retrieved {len(params)} request parameter(s).")
+
+
+		# invalid.
+		else:
+			raise exceptions.InvalidUsage(f"The parameters parameter must be [str, String, list, Array, dict, Dictionary] not [{dictionary.__class__.__name__}].")
+		
+		#
+
+	# check parameter's values.
+	def check(self, 
+		# the parameters (dict) (#1).
+		parameters={"parameter":None}, 
+		# the recognizer value for when the parameters are supposed to be empty.
+		default=None, 
+		# the traceback id.
+		traceback=None,
+	):
+
+		# single.
+		if isinstance(parameters, tuple):
+			name,parameter = parameters
+			if parameter == default: 
+				if traceback != None:
+					return r3sponse.error(f"{traceback}: Define parameter [{name}].")
+				else:
+					return r3sponse.error(f"Define parameter [{name}].")
+			if ":" in name:
+				name,formats = name.split(":")
+				while True:
+					if " " in formats: formats = formats.replace(" ","")
+					else: break
+				formats = formats.split(",")
+				param_format = Formats.get(parameter, serialize=True)
+				if param_format not in formats:
+					return r3sponse.error(f"Incorrect parameter [{name}:{param_format}] format, correct format(s): {Array(path=False, array=formats).string(joiner=', ')}.")
+			return r3sponse.success(f"Succesfully checked parameter [{name}].")
+
+		# recursive.
+		elif isinstance(parameters, (dict,Dictionary,ResponseObject)):
+			for id, value in parameters.items():
+				response = self.check(parameters=(id, value), default=default, traceback=traceback)
+				if response["error"] != None: return response
+			return r3sponse.success(f"Succesfully checked {len(parameters)} parameter(s).")
+		# invalid.
+		else: 
+			raise exceptions.InstanceError(f"Parameter [parameters] requires to be a [dict, Dictionary, tuple] not [{parameters.__class__.__name__}].")
+
+		#
+
+	#
+
+# teh response & parameters object class.
 class ResponseObject(object):
 	def __init__(self, 
 		# the response attributes.
@@ -568,20 +647,20 @@ class ResponseObject(object):
 	# support '>=' & '>' operator.
 	def __gt__(self, response):
 		if not isinstance(directory, self.__class__):
-			raise exceptions.FormatError(f"Can not compare object {self.__class__} & {response.__class__}.")
+			raise exceptions.InstanceError(f"Can not compare object {self.__class__} & {response.__class__}.")
 		return len(self) > len(response)
 	def __ge__(self, response):
 		if not isinstance(response, self.__class__):
-			raise exceptions.FormatError(f"Can not compare object {self.__class__} & {response.__class__}.")
+			raise exceptions.InstanceError(f"Can not compare object {self.__class__} & {response.__class__}.")
 		return len(self) >= len(response)
 	# support '<=' & '<' operator.
 	def __lt__(self, response):
 		if not isinstance(response, self.__class__):
-			raise exceptions.FormatError(f"Can not compare object {self.__class__} & {response.__class__}.")
+			raise exceptions.InstanceError(f"Can not compare object {self.__class__} & {response.__class__}.")
 		return len(self) < len(response)
 	def __le__(self, response):
 		if not isinstance(response, self.__class__):
-			raise exceptions.FormatError(f"Can not compare object {self.__class__} & {response.__class__}.")
+			raise exceptions.InstanceError(f"Can not compare object {self.__class__} & {response.__class__}.")
 		return len(self) <= len(response)
 	# support '==' & '!=' operator.
 	def __eq__(self, dictionary):
@@ -645,4 +724,5 @@ class ResponseObject(object):
 		return self.dict()
 
 # initialized objects.
+# must be initialized as r3sponse since Parameters requires it.
 r3sponse = R3sponse()
